Package: popEpi
Title: Functions for Epidemiological Analysis using Population Data
Version: 0.4.14
Authors@R: c(
    person("Joonas", "Miettinen", , "joonas.miettinen@cancer.fi", role = c("cre", "aut"),
           comment = c(ORCID = "0000-0001-8624-6754")),
    person("Matti", "Rantanen", , "matti.rantanen@statfinn.com", role = "aut"),
    person("Karri", "Seppa", , "karri.seppa@cancer.fi", role = "ctb")
  )
Author: Joonas Miettinen [cre, aut] (ORCID:
    <https://orcid.org/0000-0001-8624-6754>),
  Matti Rantanen [aut],
  Karri Seppa [ctb]
Maintainer: Joonas Miettinen <joonas.miettinen@cancer.fi>
Description: Enables computation of epidemiological statistics, including
    those where counts or mortality rates of the reference population are
    used.  Currently supported: excess hazard models (Dickman, Sloggett,
    Hills, and Hakulinen (2012) <doi:10.1002/sim.1597>), rates, mean
    survival times, relative/net survival (in particular the Ederer II
    (Ederer and Heise (1959)) and Pohar Perme (Pohar Perme, Stare, and
    Esteve (2012) <doi:10.1111/j.1541-0420.2011.01640.x>) estimators), and
    standardized incidence and mortality ratios, all of which can be
    easily adjusted for by covariates such as age. Fast splitting and
    aggregation of 'Lexis' objects (from package 'Epi') and other
    computations achieved using 'data.table'.
License: MIT + file LICENSE
URL: https://github.com/FinnishCancerRegistry/popEpi
BugReports: https://github.com/FinnishCancerRegistry/popEpi/issues
Depends: R (>= 3.2.0)
Imports: data.table (>= 1.10.4), Epi (>= 2.0), splines, survival
Suggests: covr, knitr, mstate, relsurv, rmarkdown, roxygen2, testthat
VignetteBuilder: knitr
ByteCompile: true
Encoding: UTF-8
Language: en-GB
LazyData: true
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-02-12 11:10:07 UTC; joonas.miettinen
Repository: CRAN
Date/Publication: 2026-02-12 14:00:02 UTC
Built: R 4.6.0; ; 2026-04-29 06:28:28 UTC; unix
