# Generated by roxygen2: do not edit by hand

S3method("[",aggre)
S3method("[",rate)
S3method("[",sir)
S3method("[",survmean)
S3method("[",survtab)
S3method("[",yrs)
S3method(as.Date,yrs)
S3method(as.aggre,data.frame)
S3method(as.aggre,data.table)
S3method(as.aggre,default)
S3method(as.yrs,Date)
S3method(as.yrs,default)
S3method(coef,sir)
S3method(confint,sir)
S3method(formula,survmean)
S3method(formula,survtab)
S3method(getCall,rate)
S3method(getCall,sir)
S3method(getCall,survmean)
S3method(getCall,survtab)
S3method(get_breaks,Lexis)
S3method(get_breaks,aggre)
S3method(get_breaks,default)
S3method(get_breaks,survtab)
S3method(lines,sirspline)
S3method(lines,survmean)
S3method(lines,survtab)
S3method(plot,rate)
S3method(plot,sir)
S3method(plot,sirspline)
S3method(plot,survmean)
S3method(plot,survtab)
S3method(print,aggre)
S3method(print,rate)
S3method(print,sir)
S3method(print,sirspline)
S3method(print,survtab)
S3method(print,yrs)
S3method(select_breaks,Lexis)
S3method(select_breaks,aggre)
S3method(select_breaks,default)
S3method(subset,aggre)
S3method(subset,rate)
S3method(subset,survmean)
S3method(subset,survtab)
S3method(summary,aggre)
S3method(summary,survtab)
export(Lexis_fpa)
export(RPL)
export(Surv)
export(adjust)
export(aggre)
export(all_names_present)
export(array_to_long_df)
export(array_to_long_dt)
export(array_to_ratetable)
export(as.aggre)
export(cast_simple)
export(cut_bound)
export(expr.by.cj)
export(fac2num)
export(get.yrs)
export(is.Date)
export(is_leap_year)
export(lexpand)
export(long_df_to_array)
export(long_df_to_ratetable)
export(long_dt_to_array)
export(long_dt_to_ratetable)
export(lower_bound)
export(ltable)
export(na2zero)
export(poisson.ci)
export(prepExpo)
export(rate)
export(rate_ratio)
export(ratetable_to_array)
export(ratetable_to_long_df)
export(ratetable_to_long_dt)
export(relpois)
export(relpois_ag)
export(robust_values)
export(rpcurve)
export(setaggre)
export(setcolsnull)
export(sir)
export(sir_ag)
export(sir_exp)
export(sir_lex)
export(sir_ratio)
export(sirspline)
export(splitLexisDT)
export(splitMulti)
export(survmean)
export(survtab)
export(survtab_ag)
export(try2int)
import(Epi)
import(data.table)
import(grDevices)
import(graphics)
import(splines)
import(stats)
importFrom(data.table,copy)
importFrom(data.table,is.data.table)
importFrom(data.table,setDT)
importFrom(data.table,setattr)
importFrom(survival,Surv)
