% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3_definitions.R
\name{print.rate}
\alias{print.rate}
\title{Print an rate object}
\usage{
\method{print}{rate}(x, subset = NULL, ...)
}
\arguments{
\item{x}{an \code{rate} object}

\item{subset}{a logical condition to subset results table by
before printing; use this to limit to a certain stratum. E.g.
\code{subset = sex == "female"}}

\item{...}{arguments for data.tables print method, e.g. row.names = FALSE suppresses row numbers.}
}
\value{
Always returns \code{NULL} invisibly.
This function is called for its side effects.
}
\description{
Print method function for \code{rate} objects; see
\verb{[rate]}.
}
\author{
Matti Rantanen
}
