% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_document.R
\name{stdpop101}
\alias{stdpop101}
\title{World standard population by 1 year age groups from 1 to 101. Sums to 100 000.}
\format{
data.table with columns
\itemize{
 \item \code{world_std} weight that sums to 100000 (numeric)
 \item \code{agegroup} age group from 1 to 101 (numeric)
}
}
\source{
Standard population is from:
\href{https://seer.cancer.gov/stdpopulations/stdpop.singleages.html}{world standard population "101of1"}
}
\description{
World standard population by 1 year age groups from 1 to 101. Sums to 100 000.
}
\seealso{
Other popEpi data: 
\code{\link{ICSS}},
\code{\link{meanpop_fi}},
\code{\link{popmort}},
\code{\link{sibr}},
\code{\link{sire}},
\code{\link{stdpop18}}

Other weights: 
\code{\link{ICSS}},
\code{\link{direct_standardization}},
\code{\link{stdpop18}}
}
\concept{popEpi data}
\concept{weights}
