% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_document.R
\name{stdpop18}
\alias{stdpop18}
\title{Standard populations from 2000: world, Europe and Nordic.}
\format{
data.table with columns
\itemize{
\item \code{agegroup}, age group in 18 categories (character)
\item \code{world}, World 2000 standard population (numeric)
\item \code{europe}, European standard population (numeric)
\item \code{nordic}, Nordic standard population (numeric)
}
}
\source{
Nordcan, 2000
}
\description{
World, European, and Nordic standard populations by 18 age categories.
Sums to 100000.
}
\seealso{
Other popEpi data: 
\code{\link{ICSS}},
\code{\link{meanpop_fi}},
\code{\link{popmort}},
\code{\link{sibr}},
\code{\link{sire}},
\code{\link{stdpop101}}

Other weights: 
\code{\link{ICSS}},
\code{\link{direct_standardization}},
\code{\link{stdpop101}}
}
\concept{popEpi data}
\concept{weights}
