% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/relative_poisson_net_survival.R
\docType{data}
\name{RPL}
\alias{RPL}
\title{Relative Poisson family object}
\format{
A list very similar to that created by \code{poisson()}.
}
\usage{
RPL
}
\description{
A family object for GLM fitting of relative Poisson models
}
\seealso{
Other relpois functions: 
\code{\link{relpois}()},
\code{\link{relpois_ag}()},
\code{\link{rpcurve}()}
}
\author{
Karri Seppa
}
\concept{relpois functions}
\keyword{datasets}
