% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_document.R
\name{sibr}
\alias{sibr}
\title{sibr - a simulated cohort of Finnish female breast cancer patients}
\format{
data.table with columns
\itemize{
\item sex - gender of the patient (1 = female)
\item bi_date - date of birth
\item dg_date - date of cancer diagnosis
\item ex_date - date of exit from follow-up (death or censoring)
\item status  - status of the person at exit; 0 alive; 1 dead due to pertinent cancer; 2 dead due to other causes
\item dg_age  - age at diagnosis expressed as fractional years
}
}
\source{
The Finnish Cancer Registry
}
\description{
\code{sibr} is a simulated cohort pertaining female Finnish breast cancer patients
diagnosed between 1993-2012. Instead of actual original dates, the dates are masked
via modest randomization within several time windows. The dataset is additionally
a random sample of 10 000 cases from the pertaining time window.
}
\details{
The closing date for the pertinent data was 2012-12-31, meaning status information was
available only up to that point --- hence the maximum possible \code{ex_date} is \code{2012-12-31}.
}
\seealso{
Other popEpi data: 
\code{\link{ICSS}},
\code{\link{meanpop_fi}},
\code{\link{popmort}},
\code{\link{sire}},
\code{\link{stdpop101}},
\code{\link{stdpop18}}

Other survival data: 
\code{\link{sire}}
}
\author{
Karri Seppa
}
\concept{popEpi data}
\concept{survival data}
