% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3_definitions.R
\name{plot.rate}
\alias{plot.rate}
\title{plot method for rate object}
\usage{
\method{plot}{rate}(x, conf.int = TRUE, eps = 0.2, left.margin, xlim, ...)
}
\arguments{
\item{x}{a rate object (see \code{\link{rate}})}

\item{conf.int}{logical; default TRUE draws the confidence intervals}

\item{eps}{is the height of the ending of the error bars}

\item{left.margin}{set a custom left margin for long variable names. Function
tries to do it by default.}

\item{xlim}{change the x-axis location}

\item{...}{arguments passed on to graphical functions points and segment 
(e.g. \code{col}, \code{lwd}, \code{pch} and \code{cex})}
}
\value{
Always returns `NULL` invisibly.
This function is called for its side effects.
}
\description{
Plot rate estimates with confidence intervals lines using R base graphics
}
\details{
This is limited explanatory tool but most graphical 
parameters are user adjustable.
}
\author{
Matti Rantanen
}
