% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utility_functions.R
\name{lower_bound}
\alias{lower_bound}
\title{Return lower_bound value from char string (20,30]}
\usage{
lower_bound(cut)
}
\arguments{
\item{cut}{is a character vector of elements "(20,60]"}
}
\value{
A numeric vector.
}
\description{
selects lowest values of each factor after cut() based
on that the value starts from index 2 and end in comma ",".
}
\author{
Matti Rantanen
}
