% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3_definitions.R
\name{summary.aggre}
\alias{summary.aggre}
\title{Summarize an \code{aggre} Object}
\usage{
\method{summary}{aggre}(object, by = NULL, subset = NULL, ...)
}
\arguments{
\item{object}{an \code{aggre} object}

\item{by}{list of columns to summarize by - e.g. \code{list(V1, V2)}
where \code{V1} and \code{V2} are columns in the data.}

\item{subset}{a logical condition to subset results table by
before summarizing; use this to limit to a certain stratum. E.g.
\code{subset = sex == "male"}}

\item{...}{unused}
}
\value{
Returns a `data.table` --- a further aggregated version of `object`.
}
\description{
\code{summary} method function for \code{aggre} objects; see
\code{\link{as.aggre}} and \code{\link{aggre}}.
}
\seealso{
Other aggregation functions: 
\code{\link{aggre}()},
\code{\link{as.aggre}()},
\code{\link{lexpand}()},
\code{\link{setaggre}()}
}
\author{
Joonas Miettinen
}
\concept{aggregation functions}
