% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utility_functions.R
\name{cut_bound}
\alias{cut_bound}
\title{Change output values from cut(..., labels = NULL) output}
\usage{
cut_bound(t, factor = TRUE)
}
\arguments{
\item{t}{is a character vector of elements, e.g. "(20,60]"}

\item{factor}{logical; TRUE returns informative character string, FALSE numeric (left value)}
}
\value{
If \code{factor = TRUE}, returns a character vector; else returns a numeric
vector.
}
\description{
Selects lowest values of each factor after cut() based
on the assumption that the value starts from index 2 and end in comma ",".
}
\details{
type = 'factor': "[50,52)" -> "50-51" OR "[50,51)" -> "50"

type = 'numeric': lowest bound in numeric.
}
\examples{
cut_bound("[1900, 1910)") ## "1900-1909"
}
\author{
Matti Rantanen
}
