% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utility_functions.R
\name{is.Date}
\alias{is.Date}
\title{Test if object is a \code{Date} object}
\usage{
is.Date(obj)
}
\arguments{
\item{obj}{object to test on}
}
\value{
\code{TRUE} if \code{obj} is of class \code{"Date"} or \code{"IDate"}.
}
\description{
Test whether \code{obj} inherits one of \code{Date} or \code{IDate}.
}
\seealso{
\verb{[get.yrs]}, \verb{[is_leap_year]}, \verb{[as.Date]}
}
\author{
Joonas Miettinen
}
